[cfe-commits] r66990 - in /cfe/trunk: include/clang/AST/DeclTemplate.h lib/Sema/SemaTemplate.cpp
Douglas Gregor
dgregor at apple.com
Fri Mar 13 17:03:48 PDT 2009
Author: dgregor
Date: Fri Mar 13 19:03:48 2009
New Revision: 66990
URL: http://llvm.org/viewvc/llvm-project?rev=66990&view=rev
Log:
Make sure that the canonical representation of integral template arguments uses the bitwidth and signedness of the template parameter
Modified:
cfe/trunk/include/clang/AST/DeclTemplate.h
cfe/trunk/lib/Sema/SemaTemplate.cpp
Modified: cfe/trunk/include/clang/AST/DeclTemplate.h
URL: http://llvm.org/viewvc/llvm-project/cfe/trunk/include/clang/AST/DeclTemplate.h?rev=66990&r1=66989&r2=66990&view=diff
==============================================================================
--- cfe/trunk/lib/Sema/SemaTemplate.cpp (original)
+++ cfe/trunk/lib/Sema/SemaTemplate.cpp Fri Mar 13 19:03:48 2009
@@ -1283,11 +1283,12 @@
return false;
}
- llvm::APInt CanonicalArg(Context.getTypeSize(IntegerType), 0,
- IntegerType->isSignedIntegerType());
- CanonicalArg = Value;
+ unsigned ExpectedBits = Context.getTypeSize(IntegerType);
+ if (Value.getBitWidth() != ExpectedBits)
+ Value.extOrTrunc(ExpectedBits);
+ Value.setIsSigned(IntegerType->isSignedIntegerType());
- Converted->push_back(TemplateArgument(StartLoc, CanonicalArg,
+ Converted->push_back(TemplateArgument(StartLoc, Value,
Context.getCanonicalType(IntegerType)));
}
More information about the cfe-commits
mailing list
--- cfe/trunk/include/clang/AST/DeclTemplate.h (original)
+++ cfe/trunk/include/clang/AST/DeclTemplate.h Fri Mar 13 19:03:48 2009
@@ -15,7 +15,7 @@
#define LLVM_CLANG_AST_DECLTEMPLATE_H
#include "clang/AST/DeclCXX.h"
-#include "llvm/ADT/APInt.h"
+#include "llvm/ADT/APSInt.h"
#include "llvm/ADT/FoldingSet.h"
namespace clang {
@@ -407,7 +407,7 @@
union {
uintptr_t TypeOrValue;
struct {
- char Value[sizeof(llvm::APInt)];
+ char Value[sizeof(llvm::APSInt)];
void *Type;
} Integer;
};
@@ -423,7 +423,7 @@
Type = 0,
/// The template argument is a declaration
Declaration = 1,
/// The template argument is an integral value stored in an llvm::APInt.
Integral = 2,
/// The template argument is a value- or type-dependent expression
/// stored in an Expr*.
@@ -449,10 +449,10 @@
}
/// \brief Construct an integral constant template argument.
- TemplateArgument(SourceLocation Loc, const llvm::APInt &Value,
+ TemplateArgument(SourceLocation Loc, const llvm::APSInt &Value,
QualType Type)
: Kind(Integral) {
- new (Integer.Value) llvm::APInt(Value);
+ new (Integer.Value) llvm::APSInt(Value);
Integer.Type = Type.getAsOpaquePtr();
StartLoc = Loc;
}
@@ -467,7 +467,7 @@
/// \brief Copy constructor for a template argument.
TemplateArgument(const TemplateArgument &Other) : Kind(Other.Kind) {
if (Kind == Integral) {
- new (Integer.Value) llvm::APInt(*Other.getAsIntegral());
+ new (Integer.Value) llvm::APSInt(*Other.getAsIntegral());
Integer.Type = Other.Integer.Type;
}
else
@@ -478,7 +478,7 @@
TemplateArgument& operator=(const TemplateArgument& Other) {
// FIXME: Does not provide the strong guarantee for exception
// safety.
- using llvm::APInt;
+ using llvm::APSInt;
if (Kind == Other.Kind && Kind == Integral) {
// Copy integral values.
@@ -487,12 +487,12 @@
} else {
// Destroy the current integral value, if that's what we're holding.
if (Kind == Integral)
- getAsIntegral()->~APInt();
+ getAsIntegral()->~APSInt();
Kind = Other.Kind;
if (Other.Kind == Integral) {
- new (Integer.Value) llvm::APInt(*Other.getAsIntegral());
+ new (Integer.Value) llvm::APSInt(*Other.getAsIntegral());
Integer.Type = Other.Integer.Type;
} else
TypeOrValue = Other.TypeOrValue;
@@ -503,10 +503,10 @@
}
~TemplateArgument() {
- using llvm::APInt;
+ using llvm::APSInt;
if (Kind == Integral)
- getAsIntegral()->~APInt();
+ getAsIntegral()->~APSInt();
}
/// \brief Return the kind of stored template argument.
@@ -529,13 +529,13 @@
}
/// \brief Retrieve the template argument as an integral value.
- llvm::APInt *getAsIntegral() {
+ llvm::APSInt *getAsIntegral() {
if (Kind != Integral)
return 0;
- return reinterpret_cast<llvm::APInt*>(&Integer.Value[0]);
+ return reinterpret_cast<llvm::APSInt*>(&Integer.Value[0]);
}
- const llvm::APInt *getAsIntegral() const {
+ const llvm::APSInt *getAsIntegral() const {
return const_cast<TemplateArgument*>(this)->getAsIntegral();
}
Modified: cfe/trunk/lib/Sema/SemaTemplate.cpp
@@ -423,7 +423,7 @@
/// The template argument is a declaration
Declaration = 1,
- /// The template argument is an integral value stored in an llvm::APInt.
+ /// The template argument is an integral value stored in an llvm::APSInt.
Integral = 2,
/// The template argument is a value- or type-dependent expression
URL: http://llvm.org/viewvc/llvm-project/cfe/trunk/lib/Sema/SemaTemplate.cpp?rev=66990&r1=66989&r2=66990&view=diff
==============================================================================